@@ -14,9 +14,11 @@ import sys
 import time
 import traceback
 
+from gunicorn.errors import HaltServer
 from gunicorn.pidfile import Pidfile
 from gunicorn.sock import create_socket
 from gunicorn import util
+from gunicorn import __version__
 
 class Arbiter(object):
     """
@@ -48,32 +50,22 @@ class Arbiter(object):
     )
     
     def __init__(self, app):
-        self.app = app
-        self.cfg = app.cfg
-        
-        if self.cfg.preload_app:
-            self.app.wsgi()
-
         self.log = logging.getLogger(__name__)
-
-        self.address = self.cfg.address
-        self.num_workers = self.cfg.workers
-        self.debug = self.cfg.debug
-        self.timeout = self.cfg.timeout
-        self.proc_name = self.cfg.proc_name
-        self.worker_class = self.cfg.worker_class
-
+        self.log.info("Starting gunicorn %s" % __version__)
+        
+        self.setup(app)
+        
         self.pidfile = None
         self.worker_age = 0
         self.reexec_pid = 0
         self.master_name = "Master"
- 
+        
         # get current path, try to use PWD env first
         try:
-            a = os.stat(os.environ('PWD'))
+            a = os.stat(os.environ['PWD'])
             b = os.stat(os.getcwd())
             if a.ino == b.ino and a.dev == b.dev:
-                cwd = os.environ('PWD')
+                cwd = os.environ['PWD']
             else:
                 cwd = os.getcwd()
         except:
@@ -88,6 +80,27 @@ class Arbiter(object):
             "cwd": cwd,
             0: sys.executable
         }
+        
+    def setup(self, app):
+        self.app = app
+        self.cfg = app.cfg
+        self.address = self.cfg.address
+        self.num_workers = self.cfg.workers
+        self.debug = self.cfg.debug
+        self.timeout = self.cfg.timeout
+        self.proc_name = self.cfg.proc_name
+        self.worker_class = self.cfg.worker_class
+        
+        if self.cfg.debug:
+            self.log.debug("Current configuration:")
+            for config, value in sorted(self.cfg.settings.iteritems()):
+                self.log.debug("  %s: %s" % (config, value.value))
+        
+        if self.cfg.preload_app:
+            if not self.cfg.debug:
+                self.app.wsgi()
+            else:
+                self.log.warning("debug mode: app isn't preloaded.")
 
     def start(self):
         """\
@@ -95,12 +108,13 @@ class Arbiter(object):
         """
         self.pid = os.getpid()
         self.init_signals()
-        self.LISTENER = create_socket(self.cfg)
+        if not self.LISTENER:
+            self.LISTENER = create_socket(self.cfg)
         
         if self.cfg.pidfile is not None:
             self.pidfile = Pidfile(self.cfg.pidfile)
             self.pidfile.create(self.pid)
-        self.log.info("Arbiter booted")
+        self.log.debug("Arbiter booted")
         self.log.info("Listening at: %s" % self.LISTENER)
         self.cfg.when_ready(self)
     
@@ -110,7 +124,7 @@ class Arbiter(object):
         are queued. Child signals only wake up the master.
         """
         if self.PIPE:
-            map(lambda p: p.close(), self.PIPE)
+            map(lambda p: os.close(p), self.PIPE)
         self.PIPE = pair = os.pipe()
         map(util.set_non_blocking, pair)
         map(util.close_on_exec, pair)
@@ -150,11 +164,15 @@ class Arbiter(object):
                     continue
                 self.log.info("Handling signal: %s" % signame)
                 handler()  
-                self.wakeup()   
+                self.wakeup()
             except StopIteration:
-                break
+                self.halt()
             except KeyboardInterrupt:
-                break
+                self.halt()
+            except HaltServer, inst:
+                self.halt(reason=inst.reason, exit_status=inst.exit_status)
+            except SystemExit:
+                raise
             except Exception:
                 self.log.info("Unhandled exception in main loop:\n%s" %  
                             traceback.format_exc())
@@ -163,12 +181,6 @@ class Arbiter(object):
                     self.pidfile.unlink()
                 sys.exit(-1)
 
-        self.stop()
-        self.log.info("Shutting down: %s" % self.master_name)
-        if self.pidfile is not None:
-            self.pidfile.unlink()
-        sys.exit(0)
-        
     def handle_chld(self, sig, frame):
         "SIGCHLD handling"
         self.wakeup()
@@ -177,12 +189,12 @@ class Arbiter(object):
     def handle_hup(self):
         """\
         HUP handling.
-        Entirely reloading the application including gracefully
-        restart the workers and rereading the configuration.
+        - Reload configuration
+        - Start the new worker processes with a new configuration
+        - Gracefully shutdown the old worker processes
         """
         self.log.info("Hang up: %s" % self.master_name)
-        self.reexec()
-        raise StopIteration
+        self.reload()
         
     def handle_quit(self):
         "SIGQUIT handling"
@@ -236,7 +248,7 @@ class Arbiter(object):
         "SIGWINCH handling"
         if os.getppid() == 1 or os.getpgrp() != os.getpid():
             self.logger.info("graceful stop of workers")
-            self.kill_workers(True)
+            self.kill_workers(signal.SIGQUIT)
         else:
             self.log.info("SIGWINCH ignored. Not daemonized")
     
@@ -250,6 +262,16 @@ class Arbiter(object):
             if e.errno not in [errno.EAGAIN, errno.EINTR]:
                 raise
                     
+    def halt(self, reason=None, exit_status=0):
+        """ halt arbiter """
+        self.stop()
+        self.log.info("Shutting down: %s" % self.master_name)
+        if reason is not None:
+            self.log.info("Reason: %s" % reason)
+        if self.pidfile is not None:
+            self.pidfile.unlink()
+        sys.exit(exit_status)
+        
     def sleep(self):
         """\
         Sleep until PIPE is readable or we timeout.
@@ -269,6 +291,7 @@ class Arbiter(object):
                 raise
         except KeyboardInterrupt:
             sys.exit()
+            
     
     def stop(self, graceful=True):
         """\
@@ -282,7 +305,7 @@ class Arbiter(object):
         if not graceful:
             sig = signal.SIGTERM
         limit = time.time() + self.timeout
-        while self.WORKERS or time.time() > limit:
+        while self.WORKERS and time.time() < limit:
             self.kill_workers(sig)
             time.sleep(0.1)
             self.reap_workers()
@@ -304,7 +327,39 @@ class Arbiter(object):
         os.chdir(self.START_CTX['cwd'])
         self.cfg.pre_exec(self)
         os.execvpe(self.START_CTX[0], self.START_CTX['args'], os.environ)
+        
+    def reload(self):
+        old_address = self.cfg.address
 
+        # reload conf
+        self.app.reload()
+        self.setup(self.app)
+
+        # do we need to change listener ?
+        if old_address != self.cfg.address:
+            self.LISTENER.close()
+            self.LISTENER = create_socket(self.cfg)
+            self.log.info("Listening at: %s" % self.LISTENER)    
+
+        # spawn new workers with new app & conf
+        for i in range(self.app.cfg.workers):
+            self.spawn_worker()
+        
+        # unlink pidfile
+        if self.pidfile is not None:
+            self.pidfile.unlink()
+
+        # create new pidfile
+        if self.cfg.pidfile is not None:
+            self.pidfile = Pidfile(self.cfg.pidfile)
+            self.pidfile.create(self.pid)
+            
+        # set new proc_name
+        util._setproctitle("master [%s]" % self.proc_name)
+        
+        # manage workers
+        self.manage_workers()
+        
     def murder_workers(self):
         """\
         Kill unused/idle workers
@@ -336,7 +391,8 @@ class Arbiter(object):
                     # to avoid infinite start/stop cycles.
                     exitcode = status >> 8
                     if exitcode == self.WORKER_BOOT_ERROR:
-                        raise StopIteration
+                        reason = "Worker failed to boot."
+                        raise HaltServer(reason, self.WORKER_BOOT_ERROR)
                     worker = self.WORKERS.pop(wpid, None)
                     if not worker:
                         continue
@@ -361,6 +417,39 @@ class Arbiter(object):
                     pid, age = wpid, worker.age
             self.kill_worker(pid, signal.SIGQUIT)
             
+    def spawn_worker(self):
+        self.worker_age += 1
+        worker = self.worker_class(self.worker_age, self.pid, self.LISTENER,
+                                    self.app, self.timeout/2.0, self.cfg)
+        self.cfg.pre_fork(self, worker)
+        pid = os.fork()
+        if pid != 0:
+            self.WORKERS[pid] = worker
+            return
+
+        # Process Child
+        worker_pid = os.getpid()
+        try:
+            util._setproctitle("worker [%s]" % self.proc_name)
+            self.log.info("Booting worker with pid: %s" % worker_pid)
+            self.cfg.post_fork(self, worker)
+            worker.init_process()
+            sys.exit(0)
+        except SystemExit:
+            raise
+        except:
+            self.log.exception("Exception in worker process:")
+            if not worker.booted:
+                sys.exit(self.WORKER_BOOT_ERROR)
+            sys.exit(-1)
+        finally:
+            self.log.info("Worker exiting (pid: %s)" % worker_pid)
+            try:
+                worker.tmp.close()
+                os.unlink(worker.tmpname)
+            except:
+                pass
+        
     def spawn_workers(self):
         """\
         Spawn new workers as needed.
@@ -370,38 +459,7 @@ class Arbiter(object):
         """
         
         for i in range(self.num_workers - len(self.WORKERS.keys())):
-            self.worker_age += 1
-            worker = self.worker_class(self.worker_age, self.pid, self.LISTENER,
-                                        self.app, self.timeout/2.0, self.cfg)
-            self.cfg.pre_fork(self, worker)
-            pid = os.fork()
-            if pid != 0:
-                self.WORKERS[pid] = worker
-                continue
-
-            # Process Child
-            worker_pid = os.getpid()
-            try:
-                util._setproctitle("worker [%s]" % self.proc_name)
-                self.log.debug("Booting worker: %s (age: %s)" % (
-                                                worker_pid, self.worker_age))
-                self.cfg.post_fork(self, worker)
-                worker.init_process()
-                sys.exit(0)
-            except SystemExit:
-                raise
-            except:
-                self.log.exception("Exception in worker process:")
-                if not worker.booted:
-                    sys.exit(self.WORKER_BOOT_ERROR)
-                sys.exit(-1)
-            finally:
-                self.log.info("Worker exiting (pid: %s)" % worker_pid)
-                try:
-                    worker.tmp.close()
-                    os.unlink(worker.tmpname)
-                except:
-                    pass
+            self.spawn_worker()
 
     def kill_workers(self, sig):
         """\
@@ -426,6 +484,7 @@ class Arbiter(object):
                     worker = self.WORKERS.pop(pid)
                     worker.tmp.close()
                     os.unlink(worker.tmpname)
+                    return
                 except (KeyError, OSError):
                     return
             raise            
