@@ -10,9 +10,26 @@ import os
 import pkg_resources
 import resource
 import socket
+import sys
 import textwrap
 import time
 
+try:#python 2.6, use subprocess
+    import subprocess
+    subprocess.Popen  # trigger ImportError early
+    closefds = os.name == 'posix'
+    
+    def popen3(cmd, mode='t', bufsize=0):
+        p = subprocess.Popen(cmd, shell=True, bufsize=bufsize,
+            stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE, 
+            close_fds=closefds)
+        p.wait()
+        return (p.stdin, p.stdout, p.stderr)
+except ImportError:
+    subprocess = None
+    popen3 = os.popen3
+
+
 MAXFD = 1024
 if (hasattr(os, "devnull")):
    REDIRECT_TO = os.devnull
@@ -57,7 +74,13 @@ def load_worker_class(uri):
     else:
         components = uri.split('.')
         if len(components) == 1:
-            raise RuntimeError("arbiter uri invalid")
+            try:
+                if uri.startswith("#"):
+                    uri = uri[1:]
+                return pkg_resources.load_entry_point("gunicorn", 
+                            "gunicorn.workers", uri)
+            except ImportError: 
+                raise RuntimeError("arbiter uri invalid or not found")
         klass = components.pop(-1)
         mod = __import__('.'.join(components))
         for comp in components[1:]:
@@ -174,13 +197,9 @@ def import_app(module):
         module, obj = module, "application"
     else:
         module, obj = parts[0], parts[1]
-    mod = __import__(module)
-    parts = module.split(".")
-    for p in parts[1:]:
-        mod = getattr(mod, p, None)
-        if mod is None:
-            raise ImportError("Failed to import: %s" % module)
-    app = getattr(mod, obj, None)
+    __import__(module)
+    mod = sys.modules[module]
+    app = eval(obj, mod.__dict__)
     if app is None:
         raise ImportError("Failed to find application object: %r" % obj)
     if not callable(app):
@@ -237,4 +256,4 @@ def daemonize():
         
         os.open(REDIRECT_TO, os.O_RDWR)
         os.dup2(0, 1)
-        os.dup2(0, 2)
\ No newline at end of file
+        os.dup2(0, 2)
