@@ -3,29 +3,40 @@
 # This file is part of gunicorn released under the MIT license. 
 # See the NOTICE for more information.
 
+from __future__ import with_statement
+
 import errno
 import socket
 import traceback
 
-from gunicorn import http
-from gunicorn.http.tee import UnexpectedEOF
-from gunicorn import util
-from gunicorn.workers.base import Worker
+import gunicorn.http as http
+import gunicorn.http.wsgi as wsgi
+import gunicorn.util as util
+import gunicorn.workers.base as base
 
 ALREADY_HANDLED = object()
 
-class AsyncWorker(Worker):
+class AsyncWorker(base.Worker):
 
     def __init__(self, *args, **kwargs):
-        Worker.__init__(self, *args, **kwargs)
+        super(AsyncWorker, self).__init__(*args, **kwargs)
         self.worker_connections = self.cfg.worker_connections
     
-    def keepalive_request(self, client, addr):
-        return http.KeepAliveRequest(self.cfg, client, addr, self.address)
+    def timeout_ctx(self):
+        raise NotImplementedError()
 
     def handle(self, client, addr):
         try:
-            while self.handle_request(client, addr):
+            parser = http.RequestParser(client)
+            try:
+                while True:
+                    req = None
+                    with self.timeout_ctx():
+                        req = parser.next()
+                    if not req:
+                        break
+                    self.handle_request(req, client, addr)
+            except StopIteration:
                 pass
         except socket.error, e:
             if e[0] not in (errno.EPIPE, errno.ECONNRESET):
@@ -34,9 +45,7 @@ class AsyncWorker(Worker):
                 if e[0] == errno.ECONNRESET:
                     self.log.warn("Ignoring connection reset")
                 else:
-                    self.log.warn("Ignoring EPIPE")
-        except UnexpectedEOF:
-            self.log.exception("Client closed the connection unexpectedly.")
+                    self.log.debug("Ignoring EPIPE")
         except Exception, e:
             self.log.exception("General error processing request.")
             try:            
@@ -49,28 +58,27 @@ class AsyncWorker(Worker):
         finally:
             util.close(client)
 
-    def handle_request(self, client, addr):
-        req = self.keepalive_request(client, addr)
-        if not req:
-            return False
+    def handle_request(self, req, sock, addr):
         try:
-            environ = req.read()
-            if not environ or not req.parser.headers:
-                return False
-            respiter = self.wsgi(environ, req.start_response)
+            debug = self.cfg.debug or False
+            self.cfg.pre_request(self, req)
+            resp, environ = wsgi.create(req, sock, addr, self.address,
+                    self.cfg)
+            respiter = self.wsgi(environ, resp.start_response)
             if respiter == ALREADY_HANDLED:
                 return False
             for item in respiter:
-                req.response.write(item)
-            req.response.close()
+                resp.write(item)
+            resp.close()
             if hasattr(respiter, "close"):
                 respiter.close()
-            if req.parser.should_close:
-                return False
+            if req.should_close():
+                raise StopIteration()
+            self.cfg.post_request(self, req)
         except Exception, e:
             #Only send back traceback in HTTP in debug mode.
             if not self.debug:
                 raise
-            util.write_error(client, traceback.format_exc())
+            util.write_error(sock, traceback.format_exc())
             return False
         return True
